package org.orbstudio;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.orbstudio.common.AppKeys;
import org.orbstudio.common.SystemBase;
import org.orbstudio.editors.IDLColorManager;
import org.orbstudio.popup.actions.factory.IORB;
import org.orbstudio.popup.actions.factory.IORBFactory;
import org.orbstudio.popup.actions.factory.JacORBFactory;
import org.orbstudio.popup.actions.factory.JavaIDLFactory;
import org.orbstudio.popup.actions.factory.OpenORBFactory;
import org.orbstudio.popup.actions.factory.OrbixFactory;
import org.orbstudio.popup.actions.factory.VisiBrokerFactory;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle
 */
public class ORBStudioPlugin extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "ORBStudio";

	// The shared instance
	private static ORBStudioPlugin plugin;

	// Resource bundle.
	private ResourceBundle resourceBundle;
	private IDLColorManager colorManager;

	private String separator = System.getProperty("line.separator");

	/**
	 * The constructor
	 */
	public ORBStudioPlugin() {
		plugin = this;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);

		/*
		 * ResourceBundle
		 */
		try {
			resourceBundle = ResourceBundle.getBundle("org.orbstudio.OrbPluginResources");
		}
		catch (MissingResourceException x) {
			resourceBundle = null;
		}

		/*
		 * IDLColorManager
		 */
		colorManager = new IDLColorManager(getPreferenceStore());

		/*
		 * SystemBase
		 */
		SystemBase.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		colorManager.dispose();
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 * 
	 * @return the shared instance
	 */
	public static ORBStudioPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path
	 * 
	 * @param path
	 *        the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}

	/**
	 * getCurrentORB()
	 * 
	 * @return
	 */
	public IORB getCurrnetORB() {
		IPreferenceStore store = getPreferenceStore();
		IORBFactory factory = null;

		String orb = store.getString(AppKeys.P_ORB);

		if (orb.equals(AppKeys.ORB_JACORB)) {
			factory = new JacORBFactory();
		}
		else if (orb.equals(AppKeys.ORB_VISIBROKER)) {
			factory = new VisiBrokerFactory();
		}
		else if (orb.equals(AppKeys.ORB_OPENORB)) {
			factory = new OpenORBFactory();
		}
		else if (orb.equals(AppKeys.ORB_ORBIX)) {
			factory = new OrbixFactory();
		}
		else if (orb.equals(AppKeys.ORB_JAVAIDL)) {
			factory = new JavaIDLFactory();
		}
		return factory.createORB();
	}

	/**
	 * Returns the string from the plugin's resource bundle, or 'key' if not
	 * found.
	 * 
	 * @param key
	 * @return
	 */
	/**
	 * @param key
	 * @return
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = ORBStudioPlugin.getDefault().getResourceBundle();

		try {
			return (bundle != null) ? bundle.getString(key) : key;
		}
		catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * getPLUGIN_ID()
	 * 
	 * @return
	 */
	public static String getPLUGIN_ID() {
		return PLUGIN_ID;
	}

	/**
	 * info_log()
	 * 
	 * @param message
	 */
	public static void info_log(String message) {
		getDefault().getLog().log(new Status(IStatus.INFO, getPLUGIN_ID(), IStatus.INFO, message, null));
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	/**
	 * getIDLColorManager();
	 * 
	 * @return
	 */
	public IDLColorManager getIDLColorManager() {
		return this.colorManager;
	}

	/**
	 * getClientImplFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getClientImplFileTemplate() throws IOException {
		return this.getTemplate("/templates/client/MyServiceClientImpl.txt");
	}

	/**
	 * getSimpleIDLFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getSimpleIDLFileTemplate() throws IOException {
		return this.getTemplate("/templates/idl/simple.txt");
	}

	/**
	 * getFactoryIDLFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getFactoryIDLFileTemplate() throws IOException {
		return this.getTemplate("/templates/idl/factory.txt");
	}

	/**
	 * getAOMServerFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getAOMServerFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/AOM_Server.txt");
	}

	/**
	 * getAOMTIEServerFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getAOMTIEServerFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/AOM_ServerTIE.txt");
	}

	/**
	 * getAOMServerImplFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getAOMServerImplFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/AOM_ServerImpl.txt");
	}

	/**
	 * getAOMTIEServerImplFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getAOMTIEServerImplFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/AOM_ServerImplTIE.txt");
	}

	/**
	 * getSATServerFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getSATServerFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/SAT_Server.txt");
	}

	/**
	 * getSATServerActivatorTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getSATServerActivatorTemplate() throws IOException {
		return this.getTemplate("/templates/server/SAT_ServantActivator.txt");
	}

	/**
	 * getSATServerImplFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getSATServerImplFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/SAT_ServerImpl.txt");
	}

	/**
	 * getSATTIEServerImplFileTempate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getSATTIEServerImplFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/SAT_ServerImplTIE.txt");
	}

	/**
	 * getSLTServerFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getSLTServerFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/SLT_Server.txt");
	}

	/**
	 * getSLTServerActivatorTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getSLTServerActivatorTemplate() throws IOException {
		return this.getTemplate("/templates/server/SLT_ServantLocator.txt");
	}

	/**
	 * getSLTServerImplFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getSLTServerImplFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/SLT_ServerImpl.txt");
	}

	/**
	 * getSLTTIEServerImplFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getSLTTIEServerImplFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/SLT_ServerImplTIE.txt");
	}

	/**
	 * getDLTServerFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getDLTServerFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/DLT_Server.txt");
	}

	/**
	 * getDLTTIEServerFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getDLTTIEServerFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/DLT_ServerTIE.txt");
	}

	/**
	 * getDLTServerImplFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getDLTServerImplFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/DLT_ServerImpl.txt");
	}

	/**
	 * getDLTTIEServerImplFileTemplate()
	 * 
	 * @return
	 * @throws IOException
	 */
	public StringBuffer getDLTTIEServerImplFileTemplate() throws IOException {
		return this.getTemplate("/templates/server/DLT_ServerImplTIE.txt");
	}

	/**
	 * getTemplate()
	 * 
	 * @param file
	 * @return
	 * @throws IOException
	 */
	private StringBuffer getTemplate(String file) throws IOException {
		StringBuffer sb = new StringBuffer();
		InputStreamReader isr = null;
		InputStream is = null;
		BufferedReader br = null;
		try {
			URL url = getBundle().getEntry(file);
			if (url == null) {
				throw new IOException();
			}
			is = url.openStream();
			isr = new InputStreamReader(is);
			br = new BufferedReader(isr);

			String buf = null;
			while ((buf = br.readLine()) != null) {
				sb.append(buf);
				sb.append(separator);
			}
		}
		finally {
			try {
				br.close();
			}
			catch (Exception ex) {
			}
		}
		return sb;
	}

	/**
	 * getIconImage()
	 * 
	 * @param filename
	 * @return
	 */
	public URL getIconImage(String filename) {
		URL url = ORBStudioPlugin.getDefault().getBundle().getEntry(filename);
		return url;
	}
}
