package org.orbstudio.editors;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.StringConverter;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

public class IDLColorManager {
	private final IPreferenceStore preferenceStore;

	private final Map<RGB, Color> colorTable = new HashMap<RGB, Color>(10);
	private final Map<String, Token> tokenTable = new HashMap<String, Token>(10);

	/**
	 * Constructor
	 * 
	 * @param preferenceStore
	 */
	public IDLColorManager(IPreferenceStore preferenceStore) {
		this.preferenceStore = preferenceStore;
	}

	/**
	 * dispose()
	 */
	public void dispose() {
		Iterator e = colorTable.values().iterator();
		while (e.hasNext()) {
			((Color) e.next()).dispose();
		}
	}

	/**
	 * getColor()
	 * 
	 * @param rgb
	 * @return
	 */
	public Color getColor(RGB rgb) {
		Color color = (Color) colorTable.get(rgb);
		if (color == null) {
			color = new Color(Display.getCurrent(), rgb);
			//colorTable.put(rgb, color);
			colorTable.put(rgb, color);
		}
		return color;
	}

	/**
	 * getToken()
	 * 
	 * @param prefKey
	 * @return
	 */
	public IToken getToken(String prefKey) {
		Token token = (Token) tokenTable.get(prefKey);
		if (token == null) {
			String colorName = preferenceStore.getString(prefKey);
			RGB rgb = StringConverter.asRGB(colorName);
			token = new Token(new TextAttribute(getColor(rgb)));
			tokenTable.put(prefKey, token);
		}
		return token;
	}

	/**
	 * affectsTextPresentation()
	 * 
	 * @param event
	 * @return
	 */
	public boolean affectsTextPresentation(PropertyChangeEvent event) {
		Token token = (Token) tokenTable.get(event.getProperty());
		return (token != null);
	}

	/**
	 * handlePreferenceStoreChanged()
	 * 
	 * @param event
	 */
	public void handlePreferenceStoreChanged(PropertyChangeEvent event) {
		String prefKey = event.getProperty();
		Token token = (Token) tokenTable.get(prefKey);
		if (token != null) {
			String colorName = preferenceStore.getString(prefKey);
			RGB rgb = StringConverter.asRGB(colorName);
			token.setData(new TextAttribute(getColor(rgb)));
		}
	}
}
