package org.orbstudio.editors;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;

/**
 * @stereotype Source Viewer
 */
public class IDLConfiguration extends SourceViewerConfiguration {
	private IDLColorManager colorManager;

	/**
	 * @stereotype creates
	 */

	/**
	 * @stereotype creates
	 */

	/**
	 * @stereotype creates
	 */

	public IDLConfiguration(IDLColorManager colorManager) {
		this.colorManager = colorManager;
	}

	/**
	 * @stereotype creates
	 */
	public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer) {
		PresentationReconciler reconciler = new PresentationReconciler();

		/*
		 * Setup 'style' in a comment part.
		 */
		DefaultDamagerRepairer commentDR = new DefaultDamagerRepairer(new CommentScanner(colorManager));
		reconciler.setDamager(commentDR, IDLPartitionScanner.IDL_COMMENT);
		reconciler.setRepairer(commentDR, IDLPartitionScanner.IDL_COMMENT);

		/*
		 * Setup String literal.
		 */
		DefaultDamagerRepairer dr = new DefaultDamagerRepairer(new SingleTokenScanner(colorManager));
		reconciler.setDamager(dr, IDLPartitionScanner.IDL_STRINGLITERAL);
		reconciler.setRepairer(dr, IDLPartitionScanner.IDL_STRINGLITERAL);

		/*
		 * Setup 'style' in default content area.
		 */
		dr = new DefaultDamagerRepairer(new IDLKeywordPartitionScanner(colorManager));
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

		return reconciler;
	}

	/**
	 * getConfiguredContentTypes
	 */
	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return new String[] { IDocument.DEFAULT_CONTENT_TYPE, IDLPartitionScanner.IDL_COMMENT,
		    IDLPartitionScanner.IDL_STRINGLITERAL };
	}
}
