package org.orbstudio.editors;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.IWhitespaceDetector;
import org.eclipse.jface.text.rules.IWordDetector;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;
import org.orbstudio.common.AppKeys;

public class IDLKeywordPartitionScanner extends RuleBasedScanner {
	private final String[] KEYWORDS = { "abstract", "any", "attribute", "boolean", "case", "char", "const", "context",
	    "custom", "default", "double", "exception", "enum", "factory", "FALSE", "fixed", "float", "in", "inout",
	    "interface", "local", "long", "module", "native", "Object", "octet", "oneway", "out", "private", "public",
	    "raises", "readonly", "sequence", "short", "string", "struct", "supports", "switch", "TRUE", "truncatable",
	    "typedef", "unsigned", "union", "ValueBase", "valuetype", "void", "wchar", "wstring", };

	/**
	 * IDLKeywordPartitionScanner()
	 * 
	 * @param colorManager
	 */
	public IDLKeywordPartitionScanner(IDLColorManager colorManager) {
		List<IRule> rules = new ArrayList<IRule>();
		IToken tagToken = colorManager.getToken(AppKeys.P_IDENTCOLOR);

		WordRule wordRule = new WordRule(new IWordDetector() {
			public boolean isWordStart(char c) {
				return Character.isJavaIdentifierStart(c);
			}

			public boolean isWordPart(char c) {
				return Character.isJavaIdentifierPart(c);
			}
		});

		for (int i = 0; i < KEYWORDS.length; i++) {
			wordRule.addWord(KEYWORDS[i], tagToken);
		}

		rules.add(wordRule);
		rules.add(new WhitespaceRule(new IWhitespaceDetector() {
			public boolean isWhitespace(char character) {
				return Character.isWhitespace(character);
			}
		}));

		IRule[] result = new IRule[rules.size()];
		rules.toArray(result);
		setRules(result);
	}
}
