package org.orbstudio.editors;

import org.eclipse.jface.text.rules.EndOfLineRule;
import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;

public class IDLPartitionScanner extends RuleBasedPartitionScanner {
	public final static String IDL_DEFAULT = "__idl_default";
	public final static String IDL_COMMENT = "__idl_comment";
	public final static String IDL_STRINGLITERAL = "__idl_stringliteral";

	/**
	 * Partition scanner for commets and string literals.
	 */
	public IDLPartitionScanner() {
		IPredicateRule[] rules = new IPredicateRule[4];

		/*
		 * Comment area.
		 */
		IToken token = new Token(IDL_COMMENT);
		rules[0] = new MultiLineRule("/*", "*/", token);
		rules[1] = new EndOfLineRule("//", token);

		/*
		 * String literal area.
		 */
		token = new Token(IDL_STRINGLITERAL);
		rules[2] = new SingleLineRule("\"", "\"", token, '\\');
		rules[3] = new SingleLineRule("\'", "\'", token, '\\');

		setPredicateRules(rules);
	}
}
