package org.orbstudio.popup.actions;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import org.eclipse.core.runtime.CoreException;
import org.orbstudio.ORBStudioPlugin;
import org.orbstudio.common.SystemBase;
import org.orbstudio.common.SystemUtil;
import org.orbstudio.common.parser.idl.IDLParser;
import org.orbstudio.exception.IDLInterfaceNotFoundException;
import org.orbstudio.exception.IDLModuleNotFoundException;
import org.orbstudio.exception.IDLParseErrorException;
import org.orbstudio.wizards.server.WizardUtil;

/**
 * Action utilities class for IDL compilers.
 * 
 * @stereotype Utility
 */
public class ActionUtil {
	/**
	 * ActionUtil
	 */
	private ActionUtil() {
	}

	/**
	 * removeGeneratedFiles Clean up existing stub/skelton directories.
	 */
	public static void removeGeneratedFiles(String prjDirectory, String idlFileName) throws CoreException {
		FileInputStream fis = null;
		try {
			fis = new FileInputStream(idlFileName);
		}
		catch (FileNotFoundException ex) {
			WizardUtil.getInstance().throwCoreException("No " + idlFileName + " found.");
		}

		IDLParser idlParser = new IDLParser(fis);
		try {
			idlParser.parse();
		}
		catch (IDLParseErrorException ex) {
			ORBStudioPlugin.info_log("SKIP DELETION OF STUB/SKELTON FILES BECAUSE OF IDL PARSING ERROR: " + ex.toString());
			return;
		}
		catch (IDLInterfaceNotFoundException ex) {
			ORBStudioPlugin
			    .info_log("IDL does not have any Interface paragraph so it doesn''t remove responding directories."
			        + ex.toString());
			WizardUtil.getInstance().throwCoreException("No INTERFACE found in your idl. Clean action ignored.");
			return;
		}
		catch (IDLModuleNotFoundException ex) {
			ORBStudioPlugin.info_log("IDL does not have any Module paragraph so it doesn't remove responding directories."
			    + ex.toString());
			WizardUtil.getInstance().throwCoreException("No MODULE found in your idl. Clean action ignored.");
			return;
		}
		finally {
			try {
				fis.close();
			}
			catch (IOException ex) {
			}
		}

		String[] rootdirs = idlParser.getRootModuleNames();
		for (int i = 0; i < rootdirs.length; i++) {
			String dirPath = prjDirectory + SystemBase.getInstance().getFileSeparator() + rootdirs[i];
			File targetDirectory = new File(dirPath);
			if (targetDirectory.isDirectory()) {
				SystemUtil.removeDirectory(targetDirectory);
			}
		}
	}
}
