package org.orbstudio.popup.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchWindow;
import org.eclipse.ui.part.FileEditorInput;
import org.orbstudio.ORBStudioPlugin;
import org.orbstudio.editors.IDLEditor;
import org.orbstudio.popup.actions.factory.IORB;

/**
 * @stereotype IDL Compilation
 */
public class IDLCompileActionDelegate implements IObjectActionDelegate {
	private IWorkbench workbench;
	private Shell shell;
	private ArrayList<IFile> ifiles; // Assigne area in selectionChanged(), not here.	

	/**
	 * IDLCompileActionDelegate
	 */
	public IDLCompileActionDelegate() {
		super();

		shell = Display.getCurrent().getActiveShell();
		workbench = PlatformUI.getWorkbench();
	}

	/**
	 * setActivePart
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * run
	 */
	public void run(IAction action) {
		for (int i = 0; i < ifiles.size(); i++) {
			IFile idlFile = ifiles.get(i);
			IProject currProject = idlFile.getProject();
			_run(action, currProject, idlFile);
		}
	}

	/**
	 * _run
	 * @param action
	 * @param currProject
	 * @param idlFile
	 */
	public void _run(IAction action, IProject currProject, IFile idlFile) {
		/*
		 * Getting workbench, activepages.
		 */
		IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
		IWorkbenchPage page = window.getActivePage();

		/*
		 * Getting IProgressMonitor for store.
		 */
		IActionBars bars = ((WorkbenchWindow) window).getActionBars();
		IStatusLineManager lineManager = bars.getStatusLineManager();
		IProgressMonitor progressMonitor = lineManager.getProgressMonitor();

		String prjDirectory = null, srcDirectory = null;

		// Get basedir of the project and the source holder.
		prjDirectory = currProject.getLocation().toFile().getAbsolutePath();
		try {
			IJavaProject javaProject = JavaCore.create(currProject);
			String basesrcdir = javaProject.getPath().toFile().getPath();
			IPackageFragment[] pkgfragments = javaProject.getPackageFragments();
			for (int i = 0; i < pkgfragments.length; i++) {
				if (pkgfragments[i].getKind() == IPackageFragmentRoot.K_SOURCE) {
					IPath path = pkgfragments[i].getPath();
					srcDirectory = path.toFile().getPath().replace(basesrcdir, "");
					srcDirectory = prjDirectory + srcDirectory;
					break;
				}
			}
		}
		catch (JavaModelException ex) {
		}

		/*
		 * Finding IDL editor...
		 * Eclipse sets null value to editor when the IDL file had not been opened.
		 * In that case, open new editor setup in marker.
		 */
		IEditorInput input = new FileEditorInput(idlFile);
		IEditorPart editor = page.findEditor(input);
		if (editor == null) {
			try {
				editor = page.openEditor(input, IDLEditor.class.getName());
			}
			catch (PartInitException e) {
				IStatus status = e.getStatus();
				ErrorDialog.openError(shell, "ORB Studio Error(1)", "ORB Studio got the exception.(1)", status);
				return;
			}
		}

		/*
		 * Save the IDL file prior to IDL compile.
		 */
		if (editor.isDirty()) {
			editor.doSave(progressMonitor);
		}

		/*
		 * Clean up a directory including stub/skelton files had been generated
		 * before.
		 */
		String idlFileName = idlFile.getLocation().toFile().getAbsolutePath();
		try {
			ActionUtil.removeGeneratedFiles(srcDirectory, idlFileName);
		}
		catch (CoreException ex) {
			ORBStudioPlugin.info_log("removeGeneratedFiles failed:" + ex.toString());
			// Pass through...
		}

		Vector<String> vResult = new Vector<String>(); // An area is to store
		// resultset during the IDL compilation.

		IORB orb = ORBStudioPlugin.getDefault().getCurrnetORB();
		try {
			if (orb.execute(srcDirectory, idlFileName, vResult, false) != 0) {
				// When a return value was not zero, it does not always mean idl
				// compiler failed. So we just write down the status in log space.
				ORBStudioPlugin.info_log(vResult.toString());
			}
		}
		catch (CoreException ex) {
			IStatus status = ex.getStatus();
			ORBStudioPlugin.info_log(status.getMessage());
			ErrorDialog.openError(shell, "ORB Studio",
			  "ORB Studio cannot make your IDL Compiler run. Please check .log file in your workspace to get more detail.",
			  status);
			return;
		}
		finally {
			try {
				currProject.refreshLocal(IResource.DEPTH_INFINITE, progressMonitor);
			}
			catch (CoreException ex) {
			}
		}

		/*
		 * Error marker.
		 */
		try {
			IResource resource = (IResource) input.getAdapter(IResource.class);
			orb.parseErrorMessage(resource, vResult);
		}
		catch (CoreException ex) {
			showErrorDialog(ex);
			return;
		}
	}

	/**
	 * showErrorDialog A dialog for CoreException.
	 */
	public void showErrorDialog(CoreException ex) {
		IStatus status = ex.getStatus();
		ErrorDialog.openError(shell, "ORB Studio Error(3)", ex.getMessage(), status);
	}

	/**
	 * selectionChanged
	 */
	@SuppressWarnings("unchecked")
	public void selectionChanged(IAction action, ISelection selection) {
		IStructuredSelection structured = (IStructuredSelection) selection;
		Iterator<IStructuredSelection> iterator = (Iterator<IStructuredSelection>) structured.iterator();
		ifiles = new ArrayList<IFile>();
		while (iterator.hasNext()) {
			Object o = structured.getFirstElement();
			if (o != null && o instanceof IFile) {
				ifiles.add((IFile) iterator.next());
			}
		}
	}
}
