package org.orbstudio.popup.actions;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchWindow;

/**
 * @stereotype IDL Cleanup
 */
public class IDLCompilerCleanActionDelegate implements IObjectActionDelegate {
	private IWorkbench workbench;
	private Shell shell;
	private ArrayList<IFile> ifiles; // Assigne area in selectionChanged(), not here.

	/**
	 * IDLCompilerCleanActionDelegate
	 */
	public IDLCompilerCleanActionDelegate() {
		super();

		shell = Display.getCurrent().getActiveShell();
		workbench = PlatformUI.getWorkbench();
	}

	/**
	 * setActivePart
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * run
	 */
	public void run(IAction action) {
		for (int i = 0; i < ifiles.size(); i++) {
			IFile idlFile = ifiles.get(i);
			IProject currProject = idlFile.getProject();
			_run(action, currProject, idlFile);
		}
	}

	/**
	 * _run
	 * @param action
	 * @param currProject
	 * @param idlFile
	 */
	@SuppressWarnings("unused")
	private void _run(IAction action, IProject currProject, IFile idlFile) {
		/*
		 * Get workbench and active page.
		 */
		IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
		IWorkbenchPage page = window.getActivePage();

		/*
		 * Get IProgressMonitor for file store.
		 */
		IActionBars bars = ((WorkbenchWindow) window).getActionBars();
		IStatusLineManager lineManager = bars.getStatusLineManager();
		IProgressMonitor progressMonitor = lineManager.getProgressMonitor();

		String prjDirectory = null, srcDirectory = null;

		// Get basedir of the project and the source holder.		
		prjDirectory = currProject.getLocation().toFile().getAbsolutePath();
		try {
			IJavaProject javaProject = JavaCore.create(currProject);
			String basesrcdir = javaProject.getPath().toFile().getPath();
			IPackageFragment[] pkgfragments = javaProject.getPackageFragments();
			for (int i = 0; i < pkgfragments.length; i++) {
				if (pkgfragments[i].getKind() == IPackageFragmentRoot.K_SOURCE) {
					IPath path = pkgfragments[i].getPath();
					srcDirectory = path.toFile().getPath().replace(basesrcdir, "");
					srcDirectory = prjDirectory + srcDirectory;
					break;
				}
			}
		}
		catch (JavaModelException ex) {
		}

		String idlFileName = idlFile.getLocation().toFile().getAbsolutePath();
		try {
			ActionUtil.removeGeneratedFiles(srcDirectory, idlFileName);
		}
		catch (CoreException ex) {
			showErrorDialog(ex);
			return;
		}

		/*
		 * Synchronize project and filesystem, here.
		 */
		try {
			currProject.refreshLocal(IResource.DEPTH_INFINITE, progressMonitor);
		}
		catch (CoreException ex) {
			showErrorDialog(ex);
			return;
		}
	}

	/**
	 * showErrorDialog Dialog for handling CoreException
	 */
	public void showErrorDialog(CoreException ex) {
		IStatus status = ex.getStatus();
		String msg = "ORB Studio trapped the exception during process works...";
		ErrorDialog.openError(shell, "ORB Studio error", msg, status);
	}

	/**
	 * selectionChanged
	 */
	@SuppressWarnings("unchecked")
	public void selectionChanged(IAction action, ISelection selection) {
		IStructuredSelection structured = (IStructuredSelection) selection;
		Iterator<IStructuredSelection> iterator = (Iterator<IStructuredSelection>) structured.iterator();
		ifiles = new ArrayList<IFile>();
		while (iterator.hasNext()) {
			Object o = structured.getFirstElement();
			if (o != null && o instanceof IFile) {
				ifiles.add((IFile) iterator.next());
			}
		}
	}
}
