package org.orbstudio.popup.actions.factory;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Vector;

import org.eclipse.core.runtime.CoreException;
import org.orbstudio.wizards.server.WizardUtil;

/**
 * @stereotype utility
 */
public final class CommandExecutor {
	String commandline = "";
	Vector<String> vResult = null;

	/**
	 * CommandExecutor
	 * 
	 * @param commandline
	 */
	public CommandExecutor(String commandline) {
		this.commandline = commandline;
	}

	/**
	 * execute
	 * 
	 * @param vResult
	 * @return
	 * @throws CoreException
	 */
	public int execute(Vector<String> vResult) throws CoreException {
		int retcode = 0;
		this.vResult = vResult;

		Runtime rt = Runtime.getRuntime();

		Process proc = null;
		try {
			proc = rt.exec(commandline);
		}
		catch (IOException ex) {
			WizardUtil.getInstance().throwCoreException("CommandExecutor#execute(1): " + ex.getMessage());
		}

		// any error message?
		StreamGobbler errorGobbler = new StreamGobbler(proc.getErrorStream());

		// any output?
		StreamGobbler outputGobbler = new StreamGobbler(proc.getInputStream());

		// kick them off
		outputGobbler.start();
		errorGobbler.start();

		try {
			retcode = proc.waitFor();
		}
		catch (InterruptedException ex) {
			WizardUtil.getInstance().throwCoreException("CommandExecutor#execute(2): " + ex.getMessage());
		}

		/*
		 * Waiting for finish of threads that reads from stdout/stderr.
		 */
		while ((outputGobbler.isAlive() == true) || (errorGobbler.isAlive() == true)) {
		}
		return retcode;
	}

	private final class StreamGobbler extends Thread {
		InputStream is;
		OutputStream os;

		/**
		 * StreamGobbler
		 * 
		 * @param is
		 */
		StreamGobbler(InputStream is) {
			this(is, null);
		}

		/**
		 * StreamGobbler
		 * 
		 * @param is
		 * @param redirect
		 */
		StreamGobbler(InputStream is, OutputStream redirect) {
			this.is = is;
			this.os = redirect;
		}

		/**
		 * run
		 */
		public void run() {
			try {
				PrintWriter pw = null;
				if (os != null) {
					pw = new PrintWriter(os);
				}

				InputStreamReader isr = new InputStreamReader(is);
				BufferedReader br = new BufferedReader(isr);
				String line = null;
				while ((line = br.readLine()) != null) {
					if (pw != null) {
						pw.println(line);
					}
					vResult.add(line);
				}
				if (pw != null) {
					pw.flush();
				}
			}
			catch (IOException ioe) {
				ioe.printStackTrace();
				return;
			}
		}
	}
}
