package org.orbstudio.popup.actions.factory;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.texteditor.MarkerUtilities;
import org.orbstudio.ORBStudioPlugin;
import org.orbstudio.common.AppKeys;
import org.orbstudio.common.SystemBase;
import org.orbstudio.common.SystemUtil;

/**
 * Concrete product defines a product object to be created by the corresponding
 * concrete factory.
 */
public class OpenORBIDLImpl implements IORB {
	private String separator = System.getProperties().getProperty("line.separator");
	/**
	 * @clientCardinality 1
	 * @supplierCardinality 0..1
	 */
	private CommandExecutor commandExecutor;
	private IPreferenceStore store;
	private String ORBpropName = AppKeys.P_OPENORB_CMDLINE;
	private String ORBname = AppKeys.ORB_OPENORB;
	private String idlFileName;

	/**
	 * Constructor
	 */
	public OpenORBIDLImpl() {
		store = ORBStudioPlugin.getDefault().getPreferenceStore();
	}

	/**
	 * execute
	 */
	public int execute(String prjDirectory, String idlFileName, Vector<String> vResult, boolean bTie)
	    throws CoreException {
		this.idlFileName = idlFileName;

		/**
		 * If we work onWindows platform, adjust the file name.
		 */
		SystemBase sb = SystemBase.getInstance();
		if (sb.isWindows()) {
			idlFileName = "\"" + idlFileName + "\"";
			prjDirectory = "\"" + prjDirectory + "\"";
		}

		String command = store.getString(AppKeys.P_OPENORB_CMD);
		String commandline = store.getString(AppKeys.P_OPENORB_CMDLINE);
		command += " ";
		command += commandline;
		int retcode = -1;
		/*
		 * Build up a command line by replacing some parameters, here.
		 */
		command = SystemUtil.strReplace(command, AppKeys.MARK_IDLFILE, idlFileName);
		command = SystemUtil.strReplace(command, AppKeys.MARK_OUTPUTDIR, prjDirectory);

		commandExecutor = new CommandExecutor(command);
		retcode = commandExecutor.execute(vResult);

		return retcode;
	}

	/**
	 * parseErrorMessage Parsing output from JacORB IDL compiler.
	 */
	public void parseErrorMessage(IResource resource, Vector vResult) throws CoreException {
		SystemBase sb = SystemBase.getInstance();

		/*
		 * Clear all exising marker.
		 */
		resource.deleteMarkers("ORBStudio.IDLproblem", true, IResource.DEPTH_ZERO);
		/*
		 * OpenORB IDL To Java Compiler, (c) 2002 The Community OpenORB compile : a.idl 
		 * file:/C:/Temp/a.idl:6: Undefined identifier vid there are errors...
		 * compilation process stopped !
		 */
		idlFileName = new File(idlFileName).toURI().toString();
		if (sb.isWindows()) {
			idlFileName = idlFileName.replaceAll("%20", " ");
		}

		Iterator iterator = null;
		iterator = vResult.iterator();

		while (iterator.hasNext()) {
			String buffer = (String) iterator.next();
			String message = null;

			if (buffer.startsWith(idlFileName)) {
				message = buffer.substring(idlFileName.length() + 1);
				StringTokenizer st = new StringTokenizer(message);
				String strLineno = st.nextToken(":");

				int lineno = 0;
				try {
					strLineno = strLineno.trim();
					lineno = new Integer(strLineno);
				}
				catch (NumberFormatException ex) {
					String errmsg = "Invalid numeric found at line info: " + buffer + " ," + strLineno;
					ORBStudioPlugin.info_log(errmsg);
					IStatus status = new Status(IStatus.ERROR, "org.orbstudio", IStatus.OK, errmsg, ex);
					throw new CoreException(status);
				}
				message = st.nextToken();

				Map<String, Comparable> attributes = new HashMap<String, Comparable>();
				MarkerUtilities.setLineNumber(attributes, lineno);
				MarkerUtilities.setMessage(attributes, message);

				attributes.put(IMarker.SEVERITY, new Integer(IMarker.SEVERITY_ERROR));
				attributes.put("selection", "");

				MarkerUtilities.createMarker(resource, attributes, "ORBStudio.IDLproblem");
			}
		}
	}

	/**
	 * getORBpropname
	 */
	public String getORBpropname() {
		return ORBpropName;
	}

	/**
	 * getORBPropertiesCode Generate ORB specific property lines.
	 */
	public StringBuffer getORBPropertiesCode() {
		String TABS = "\t\t";
		StringBuffer sb = new StringBuffer();
		sb.append("Properties props = System.getProperties();" + separator);
		sb.append(TABS + "//props.setProperty(\"org.omg.CORBA.ORBClass\", \"org.openorb.CORBA.ORB\");" + separator);
		sb.append(TABS + "//props.setProperty(\"org.omg.CORBA.ORBSingletonClass\", \"org.openorb.CORBA.ORBSingleton\");"
		    + separator);
		sb.append(TABS + "// OpenORB 1.4.X" + separator);
		sb.append(TABS + "props.setProperty(\"org.omg.CORBA.ORBClass\", \"org.openorb.orb.core.ORB\");" + separator);
		sb.append(TABS + "props.setProperty(\"org.omg.CORBA.ORBSingletonClass\", \"org.openorb.orb.core.ORBSingleton\");"
		    + separator);
		return sb;
	}

	/**
	 * getORBname
	 */
	public String getORBname() {
		return ORBname;
	}
}
