package org.orbstudio.popup.actions.factory;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.texteditor.MarkerUtilities;
import org.orbstudio.ORBStudioPlugin;
import org.orbstudio.common.AppKeys;
import org.orbstudio.common.SystemBase;
import org.orbstudio.common.SystemUtil;

/**
 * Concrete product defines a product object to be created by the corresponding
 * concrete factory.
 */
public class OrbixIDLImpl implements IORB {
	private String separator = System.getProperties().getProperty("line.separator");
	private String ORBname = AppKeys.ORB_ORBIX;
	private String ORBpropName = AppKeys.P_ORBIX_CMDLINE;
	private IPreferenceStore store;
	/**
	 * @clientCardinality 1
	 * @supplierCardinality 0..1
	 */
	private CommandExecutor commandExecutor;

	/**
	 * Constructor.
	 */
	public OrbixIDLImpl() {
		store = ORBStudioPlugin.getDefault().getPreferenceStore();
	}

	/**
	 * execute
	 */
	public int execute(String prjDirectory, String idlFileName, Vector<String> vResult, boolean bTie)
	    throws CoreException {
		/**
		 * If we work onWindows platform, adjust the file name.
		 */
		SystemBase sb = SystemBase.getInstance();
		if (sb.isWindows()) {
			idlFileName = "\"" + idlFileName + "\"";
			prjDirectory = "\"" + prjDirectory + "\"";
		}

		String command = store.getString(AppKeys.P_ORBIX_CMD);
		String commandline = store.getString(AppKeys.P_ORBIX_CMDLINE);
		command += " ";
		command += commandline;
		int retcode = -1;

		/*
		 * Build up a command line by replacing some parameters, here.
		 */
		command = SystemUtil.strReplace(command, AppKeys.MARK_IDLFILE, idlFileName);
		command = SystemUtil.strReplace(command, AppKeys.MARK_OUTPUTDIR, prjDirectory);

		commandExecutor = new CommandExecutor(command);
		retcode = commandExecutor.execute(vResult);

		return retcode;
	}

	/**
	 * parseErrorMessage Parsing output from JacORB IDL compiler.
	 */
	public void parseErrorMessage(IResource resource, Vector vResult) throws CoreException {
		if (resource != null) {
			/*
			 * Clear all exising marker.
			 */
			resource.deleteMarkers("ORBStudio.IDLproblem", true, IResource.DEPTH_ZERO);

			/*
			 * Convert all messages from IDL compiler to marker object, here.
			 */
			String LINEMARK = ", line ";
			String COLLON = ":";
			Iterator iterator = vResult.iterator();
			while (iterator.hasNext()) {
				String msg = (String) iterator.next();

				int idx = msg.indexOf(LINEMARK);
				if (idx == -1) {
					continue;
				}
				int idx2 = msg.indexOf(COLLON, idx + LINEMARK.length());
				String message = msg.substring(idx2 + 1);
				String lineno = msg.substring(idx + LINEMARK.length(), idx2);

				int line = 0;
				try {
					lineno = lineno.trim();
					line = new Integer(lineno);
				}
				catch (NumberFormatException ex) {
					String errmsg = "Invalid numeric found at line info: " + msg + " ," + lineno;
					ORBStudioPlugin.info_log(errmsg);
					IStatus status = new Status(IStatus.ERROR, "org.orbstudio", IStatus.OK, errmsg, ex);
					throw new CoreException(status);
				}

				Map<String, Comparable> attributes = new HashMap<String, Comparable>();
				MarkerUtilities.setLineNumber(attributes, line);
				MarkerUtilities.setMessage(attributes, message);
				attributes.put(IMarker.SEVERITY, new Integer(IMarker.SEVERITY_ERROR));
				attributes.put("selection", "");

				MarkerUtilities.createMarker(resource, attributes, "ORBStudio.IDLproblem");
			}
		}
	}

	/**
	 * getORBpropname
	 */
	public String getORBpropname() {
		return ORBpropName;
	}

	/**
	 * getORBPropertiesCode Generate ORB specific property lines.
	 */
	public StringBuffer getORBPropertiesCode() {
		String TABS = "\t\t";
		StringBuffer sb = new StringBuffer();
		sb.append("Properties props = System.getProperties();" + separator);
		sb.append(TABS + "props.setProperty(\"org.omg.CORBA.ORBClass\", \"com.iona.corba.art.artimpl.ORBImpl\");"
		    + separator);
		sb.append(TABS
		    + "props.setProperty(\"org.omg.CORBA.ORBSingletonClass\", \"com.iona.corba.art.artimpl.ORBSingleton\");"
		    + separator);
		return sb;
	}

	/**
	 * getORBname
	 */
	public String getORBname() {
		return ORBname;
	}
}
