package org.orbstudio.popup.actions.factory;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.texteditor.MarkerUtilities;
import org.orbstudio.ORBStudioPlugin;
import org.orbstudio.common.AppKeys;
import org.orbstudio.common.SystemBase;
import org.orbstudio.common.SystemUtil;

/**
 * Concrete product defines a product object to be created by the corresponding
 * concrete factory.
 */
public class VisiBrokerIDLImpl implements IORB {
	private String separator = System.getProperties().getProperty("line.separator");
	/**
	 * @clientCardinality 1
	 * @supplierCardinality 0..1
	 */
	private CommandExecutor commandExecutor;
	private String ORBpropName = AppKeys.P_VISIBROKER_CMDLINE;
	private String ORBname = AppKeys.ORB_VISIBROKER;
	private IPreferenceStore store;
	private String idlFileName = null;

	/**
	 * Constructor
	 */
	public VisiBrokerIDLImpl() {
		store = ORBStudioPlugin.getDefault().getPreferenceStore();
	}

	/**
	 * execute
	 */
	public int execute(String prjDirectory, String idlFileName, Vector<String> vResult, boolean bTie)
	    throws CoreException {
		this.idlFileName = idlFileName;

		/**
		 * If we work onWindows platform, adjust the file name.
		 */
		SystemBase sb = SystemBase.getInstance();
		if (sb.isWindows()) {
			idlFileName = "\"" + idlFileName + "\"";
			prjDirectory = "\"" + prjDirectory + "\"";
		}

		String command = store.getString(AppKeys.P_VISIBROKER_CMD);
		String commandline = store.getString(AppKeys.P_VISIBROKER_CMDLINE);
		command += " ";
		command += commandline;
		int retcode = -1;

		/*
		 * Build up a command line by replacing some parameters, here.
		 */
		command = SystemUtil.strReplace(command, AppKeys.MARK_IDLFILE, idlFileName);
		command = SystemUtil.strReplace(command, AppKeys.MARK_OUTPUTDIR, prjDirectory);

		commandExecutor = new CommandExecutor(command);
		retcode = commandExecutor.execute(vResult);

		return retcode;
	}

	/**
	 * C:\Temp>idl2java -root_dir .\ -tie sample.idl 
	 * sample.idl: 6:voi not found(expected type name) 
	 * sample.idl: 7:abc not found (expected type name) 
	 * 2 errors
	 */
	public void parseErrorMessage(IResource resource, Vector vResult) throws CoreException {
		/*
		 * Clear all exising marker.
		 */
		resource.deleteMarkers("ORBStudio.IDLproblem", true, IResource.DEPTH_ZERO);

		Iterator iterator = null;
		iterator = vResult.iterator();

		while (iterator.hasNext()) {
			String buffer = (String) iterator.next();

			ORBStudioPlugin.info_log(buffer);

			if (buffer.startsWith(idlFileName)) {
				// ŏIDLt@C菜B':'ɂ镶񕪊s
				// hCu^[ΏۂƂȂĂ܂A܂ȂB
				buffer = buffer.substring(idlFileName.length() + 1);
				StringTokenizer st = new StringTokenizer(buffer);

				String strLineno = st.nextToken(":"); // line no.

				int lineno = 0;
				try {
					strLineno = strLineno.trim();
					lineno = new Integer(strLineno);
				}
				catch (NumberFormatException ex) {
					String errmsg = "Invalid numeric found at line info: " + buffer + " ," + strLineno;
					ORBStudioPlugin.info_log(errmsg);
					IStatus status = new Status(IStatus.ERROR, "org.orbstudio", IStatus.OK, errmsg, ex);
					throw new CoreException(status);
				}

				String message = st.nextToken(":"); // error message.

				Map<String, Comparable> attributes = new HashMap<String, Comparable>();
				MarkerUtilities.setLineNumber(attributes, lineno);
				MarkerUtilities.setMessage(attributes, message);

				attributes.put(IMarker.SEVERITY, new Integer(IMarker.SEVERITY_ERROR));
				attributes.put("selection", "");

				MarkerUtilities.createMarker(resource, attributes, "ORBStudio.IDLproblem");
			}
		}
	}

	/**
	 * getORBpropname
	 */
	public String getORBpropname() {
		return ORBpropName;
	}

	/**
	 * getORBPropertiesCode Generate ORB specific property lines.
	 */
	public StringBuffer getORBPropertiesCode() {
		String TABS = "\t\t";
		StringBuffer sb = new StringBuffer();
		sb.append("Properties props = System.getProperties();" + separator);
		sb.append(TABS + "props.setProperty(\"org.omg.CORBA.ORBClass\", \"com.inprise.vbroker.orb.ORB\");" + separator);
		sb.append(TABS + "props.setProperty(\"org.omg.CORBA.ORBSingletonClass\", \"com.inprise.vbroker.orb.ORB\");"
		    + separator);
		return sb;
	}

	/**
	 * getORBname
	 */
	public String getORBname() {
		return ORBname;
	}
}
