package org.orbstudio.wizards.client;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.orbstudio.common.parser.idl.IDLParser;
import org.orbstudio.exception.IDLFileNotFoundException;
import org.orbstudio.exception.IDLInterfaceNotFoundException;
import org.orbstudio.exception.IDLModuleNotFoundException;
import org.orbstudio.exception.IDLParseErrorException;
import org.orbstudio.exception.ProjectNotOpenedException;
import org.orbstudio.wizards.server.WizardUtil;

public class ClientWizardPage extends WizardPage {
	private IStructuredSelection selection;

	private Label containerLabel;
	private Combo idlfilesCombo;
	private Combo interfaceCombo;
	private Combo packageCombo;
	private Text implfileText;

	/**
	 * ClientWizardPage()
	 * 
	 * @param pageName
	 * @throws ProjectNotOpenedException
	 * @throws IDLFileNotFoundException
	 * @throws CoreException
	 */
	protected ClientWizardPage(IWorkbench workbench, IStructuredSelection selection) throws ProjectNotOpenedException,
	    CoreException, IDLFileNotFoundException {
		super("ClientWizard");

		this.selection = selection;

		/**
		 * Here we look up the IDL file registered in the project and check
		 * existence of the file and if the project not opened and is exist.
		 */
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IContainer container = getProjectContainer();
		if (container == null) {
			throw new ProjectNotOpenedException("No Project found.");
		}
		IProject project = root.getProject(getProjectContainer().getName());
		WizardUtil.getInstance().getIDLfiles(project);

		this.setPageComplete(false);
	}

	/**
	 * getProjectContainer
	 * 
	 * @return
	 */
	public IContainer getProjectContainer() {
		IContainer projContainer = null;

		if (selection != null && selection.isEmpty() == false && selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1) {
				return null;
			}
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				projContainer = ((IResource) obj).getProject();
			}
		}
		return projContainer;
	}

	/**
	 * getPackageFragmentRoot
	 * 
	 * @return
	 */
	public IPackageFragmentRoot getPackageFragmentRoot() {
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IProject project = root.getProject(getProjectContainer().getName());

		IJavaProject javaProject = JavaCore.create(project);
		try {
			IPackageFragmentRoot[] arrayPackageRoot = javaProject.getPackageFragmentRoots();
			for (int i = 0; i < arrayPackageRoot.length; i++) {
				if (arrayPackageRoot[i].getKind() == IPackageFragmentRoot.K_SOURCE) {
					IPackageFragmentRoot srcFolder = arrayPackageRoot[i];
					return srcFolder;
				}
			}
		}
		catch (JavaModelException e1) {
			e1.printStackTrace();
		}
		return null;
	}

	/**
	 * getPackageFragments
	 * 
	 * @return
	 */
	public IPackageFragment[] getPackageFragments() {
		ArrayList<IPackageFragment> list = new ArrayList<IPackageFragment>();
		try {
			IPackageFragmentRoot srcFolder = this.getPackageFragmentRoot();
			IJavaElement[] elements = srcFolder.getChildren();
			for (int ii = 0; ii < elements.length; ii++) {
				IPackageFragment packageFragment = (IPackageFragment) elements[ii];
				list.add(packageFragment);
			}
		}
		catch (JavaModelException e) {
			e.printStackTrace();
		}
		IPackageFragment[] res = (IPackageFragment[]) list.toArray(new IPackageFragment[list.size()]);
		return res;
	}

	/**
	 * createControl
	 */
	public void createControl(Composite parent) {
		IFile[] idlfiles = null;
		String projectName = getProjectContainer().getName();

		/*
		 * Display the name of the project.
		 */
		Composite composite = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		composite.setLayout(layout);
		layout.numColumns = 2;
		layout.verticalSpacing = 9;
		Label label = new Label(composite, SWT.NULL);
		label.setText("Project:");

		containerLabel = new Label(composite, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		containerLabel.setLayoutData(gd);
		containerLabel.setText(projectName);

		/*
		 * Detect IDL files registered in the project and register them into the
		 * combo box.
		 */
		try {
			IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
			IProject project = root.getProject(getProjectContainer().getName());

			idlfiles = WizardUtil.getInstance().getIDLfiles(project);
		}
		catch (IDLFileNotFoundException ex) {
			ex.printStackTrace();
		}
		catch (ProjectNotOpenedException ex) {
			ex.printStackTrace();
		}
		catch (CoreException ex) {
			ex.printStackTrace();
		}

		label = new Label(composite, SWT.NULL);
		label.setText("IDL filename(&m):");
		idlfilesCombo = new Combo(composite, SWT.NONE | SWT.READ_ONLY);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		idlfilesCombo.setLayoutData(gd);
		for (int i = 0; i < idlfiles.length; i++) {
			IFile idlfile = idlfiles[i];

			idlfilesCombo.add(idlfile.getFullPath().toString());
			idlfilesCombo.setData(idlfile.getFullPath().toString(), idlfile);
		}
		idlfilesCombo.addSelectionListener(new IDLfileSelection());

		label = new Label(composite, SWT.NULL);
		label.setText("Interface(&I):");
		interfaceCombo = new Combo(composite, SWT.NONE | SWT.READ_ONLY);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		interfaceCombo.setLayoutData(gd);
		interfaceCombo.addSelectionListener(new InterfaceSelection());
		interfaceCombo.setEnabled(false);
		interfaceCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				String interfaceName = interfaceCombo.getText();
				if (interfaceName.length() == 0) {
					updateStatus("Choose an interface to implement.");
					return;
				}
				updateStatus(null);
			}
		});

		/*
		 * List up packages.
		 */
		label = new Label(composite, SWT.NULL);
		label.setText("Package(&P):");
		packageCombo = new Combo(composite, SWT.NONE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		packageCombo.setLayoutData(gd);
		IPackageFragment[] packageFragments = this.getPackageFragments();
		for (int i = 0; i < packageFragments.length; i++) {
			String pkgname = packageFragments[i].getElementName();
			if (pkgname.equals("")) {
				continue;
			}
			packageCombo.add(pkgname);
			packageCombo.setData(pkgname, packageFragments[i]);
		}
		packageCombo.setEnabled(false);

		/*
		 * The name of Client impl file.
		 */
		label = new Label(composite, SWT.NULL);
		label.setText("Client classname(&C):");
		implfileText = new Text(composite, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		implfileText.setLayoutData(gd);
		implfileText.setEnabled(false);
		implfileText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				String implfileName = implfileText.getText();
				if (implfileName.length() == 0) {
					updateStatus("Enter classname.");
					return;
				}
				if (implfileName.indexOf('.') >= 0) {
					updateStatus("Please don't include package part in the classname.");
					return;
				}
				updateStatus(null);
			}
		});
		setControl(composite);
	}

	/**
	 * getPacakgeName()
	 * 
	 * @return
	 */
	public String getPackageName() {
		return packageCombo.getText();
	}

	/**
	 * getImplFileName()
	 * 
	 * @return
	 */
	public String getImplFileName() {
		return implfileText.getText();
	}

	/**
	 * getInterface()
	 * 
	 * @return
	 */
	public String getInterface() {
		return this.interfaceCombo.getText();
	}

	/**
	 * updateStatus()
	 * 
	 * @param message
	 */
	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}

	private class IDLfileSelection implements SelectionListener {
		@SuppressWarnings("unchecked")
    public void widgetSelected(SelectionEvent e) {
			Combo combo = (Combo) e.widget;
			int selidx = combo.getSelectionIndex();
			String selectedItem = combo.getItem(selidx);

			IFile idlfile = (IFile) combo.getData(selectedItem);
			File file = idlfile.getLocation().toFile();

			FileInputStream fis;
			try {
				fis = new FileInputStream(file);
			}
			catch (FileNotFoundException ex) {
				ex.printStackTrace();
				return;
			}

			IDLParser idlParser = new IDLParser(fis);
			try {
				idlParser.parse();
			}
			catch (IDLParseErrorException ex) {
				return;
			}
			catch (IDLModuleNotFoundException ex) {
				return;
			}
			catch (IDLInterfaceNotFoundException ex) {
				return;
			}
			ArrayList intfList = idlParser.getInterfaces();
			String[] list = (String[]) intfList.toArray(new String[intfList.size()]);
			interfaceCombo.setItems(list);
			interfaceCombo.setEnabled(true);

			String[] rootModuleNames = idlParser.getRootModuleNames();
			try {
				for (int i = 0; i < rootModuleNames.length; i++) {
					packageCombo.remove(rootModuleNames[i]);
				}
			}
			catch (Exception ex) {
			}
		}

		/**
		 * widgetDefaultSelected
		 */
		public void widgetDefaultSelected(SelectionEvent e) {
		}
	}

	private class InterfaceSelection implements SelectionListener {
		/**
		 * widgetSelected
		 */
		public void widgetSelected(SelectionEvent e) {
			String implfile = interfaceCombo.getText();
			int idx = implfile.lastIndexOf('.');
			if (idx >= 0) {
				implfile = implfile.substring(idx + 1);
			}
			implfileText.setText(implfile + "ClientImpl");
			implfileText.setEnabled(true);
			packageCombo.setEnabled(true);
		}

		/**
		 * widgetDefaultSelected
		 */
		public void widgetDefaultSelected(SelectionEvent e) {
		}
	}
}
